@@ -1,11 +1,13 @@
 #include <xml_sax_parser.h>
 
 int vasprintf (char **strp, const char *fmt, va_list ap);
+void vasprintf_free (void *p);
 
 static ID id_start_document, id_end_document, id_start_element, id_end_element;
 static ID id_start_element_namespace, id_end_element_namespace;
 static ID id_comment, id_characters, id_xmldecl, id_error, id_warning;
 static ID id_cdata_block, id_cAttribute;
+static ID id_processing_instruction;
 
 #define STRING_OR_NULL(str) \
    (RTEST(str) ? StringValuePtr(str) : NULL)
@@ -62,8 +64,10 @@ static void start_element(void * ctx, const xmlChar *name, const xmlChar **atts)
   int i = 0;
   if(atts) {
     while((attr = atts[i]) != NULL) {
-      rb_ary_push(attributes, NOKOGIRI_STR_NEW2(attr));
-      i++;
+      const xmlChar * val = atts[i+1];
+      VALUE value = val != NULL ? NOKOGIRI_STR_NEW2(val) : Qnil;
+      rb_ary_push(attributes, rb_ary_new3(2, NOKOGIRI_STR_NEW2(attr), value));
+      i+=2;
     }
   }
 
@@ -196,14 +200,16 @@ static void warning_func(void * ctx, const char *msg, ...)
   VALUE self = NOKOGIRI_SAX_SELF(ctx);
   VALUE doc = rb_iv_get(self, "@document");
   char * message;
+  VALUE ruby_message;
 
   va_list args;
   va_start(args, msg);
   vasprintf(&message, msg, args);
   va_end(args);
 
-  rb_funcall(doc, id_warning, 1, NOKOGIRI_STR_NEW2(message));
-  free(message);
+  ruby_message = NOKOGIRI_STR_NEW2(message);
+  vasprintf_free(message);
+  rb_funcall(doc, id_warning, 1, ruby_message);
 }
 
 static void error_func(void * ctx, const char *msg, ...)
@@ -211,14 +217,16 @@ static void error_func(void * ctx, const char *msg, ...)
   VALUE self = NOKOGIRI_SAX_SELF(ctx);
   VALUE doc = rb_iv_get(self, "@document");
   char * message;
+  VALUE ruby_message;
 
   va_list args;
   va_start(args, msg);
   vasprintf(&message, msg, args);
   va_end(args);
 
-  rb_funcall(doc, id_error, 1, NOKOGIRI_STR_NEW2(message));
-  free(message);
+  ruby_message = NOKOGIRI_STR_NEW2(message);
+  vasprintf_free(message);
+  rb_funcall(doc, id_error, 1, ruby_message);
 }
 
 static void cdata_block(void * ctx, const xmlChar * value, int len)
@@ -229,6 +237,19 @@ static void cdata_block(void * ctx, const xmlChar * value, int len)
   rb_funcall(doc, id_cdata_block, 1, string);
 }
 
+static void processing_instruction(void * ctx, const xmlChar * name, const xmlChar * content)
+{
+  VALUE self = NOKOGIRI_SAX_SELF(ctx);
+  VALUE doc = rb_iv_get(self, "@document");
+
+  rb_funcall( doc,
+              id_processing_instruction,
+              2,
+              NOKOGIRI_STR_NEW2(name),
+              NOKOGIRI_STR_NEW2(content)
+  );
+}
+
 static void deallocate(xmlSAXHandlerPtr handler)
 {
   NOKOGIRI_DEBUG_START(handler);
@@ -238,7 +259,7 @@ static void deallocate(xmlSAXHandlerPtr handler)
 
 static VALUE allocate(VALUE klass)
 {
-  xmlSAXHandlerPtr handler = calloc(1, sizeof(xmlSAXHandler));
+  xmlSAXHandlerPtr handler = calloc((size_t)1, sizeof(xmlSAXHandler));
 
   xmlSetStructuredErrorFunc(NULL, NULL);
 
@@ -253,6 +274,7 @@ static VALUE allocate(VALUE klass)
   handler->warning = warning_func;
   handler->error = error_func;
   handler->cdataBlock = cdata_block;
+  handler->processingInstruction = processing_instruction;
   handler->initialized = XML_SAX2_MAGIC;
 
   return Data_Wrap_Struct(klass, NULL, deallocate, handler);
@@ -283,4 +305,5 @@ void init_xml_sax_parser()
   id_cAttribute     = rb_intern("Attribute");
   id_start_element_namespace = rb_intern("start_element_namespace");
   id_end_element_namespace = rb_intern("end_element_namespace");
+  id_processing_instruction = rb_intern("processing_instruction");
 }
