@@ -24,6 +24,11 @@ module Nokogiri
         end
       end
 
+      def test_colons_are_not_removed
+        doc = Nokogiri::HTML::DocumentFragment.parse("<span>3:30pm</span>")
+        assert_match(/3:30/, doc.to_s)
+      end
+
       def test_parse_encoding
         fragment = "<div>hello world</div>"
         f = Nokogiri::HTML::DocumentFragment.parse fragment, 'ISO-8859-1'
@@ -74,6 +79,16 @@ module Nokogiri
         assert Nokogiri::HTML::DocumentFragment.new(@html)
       end
 
+      def test_body_fragment_should_contain_body
+        fragment = Nokogiri::HTML::DocumentFragment.parse("  <body><div>foo</div></body>")
+        assert_match(/^<body>/, fragment.to_s)
+      end
+
+      def test_nonbody_fragment_should_not_contain_body
+        fragment = Nokogiri::HTML::DocumentFragment.parse("<div>foo</div>")
+        assert_match(/^<div>/, fragment.to_s)
+      end
+
       def test_fragment_should_have_document
         fragment = Nokogiri::HTML::DocumentFragment.new(@html)
         assert_equal @html, fragment.document
@@ -123,7 +138,8 @@ module Nokogiri
       def test_html_fragment_has_outer_text
         doc = "a<div>b</div>c"
         fragment = Nokogiri::HTML::Document.new.fragment(doc)
-        if Nokogiri::VERSION_INFO['libxml']['loaded'] <= "2.6.16"
+        if Nokogiri.uses_libxml? &&
+            Nokogiri::VERSION_INFO['libxml']['loaded'] <= "2.6.16"
           assert_equal "a<div>b</div><p>c</p>", fragment.to_s
         else
           assert_equal "a<div>b</div>c", fragment.to_s
@@ -139,13 +155,13 @@ module Nokogiri
       def test_html_fragment_with_leading_whitespace
         doc = "     <div>b</div>  "
         fragment = Nokogiri::HTML::Document.new.fragment(doc)
-        assert_equal "<div>b</div>", fragment.to_s
+        assert_match %r%     <div>b</div> *%, fragment.to_s
       end
 
       def test_html_fragment_with_leading_whitespace_and_newline
         doc = "     \n<div>b</div>  "
         fragment = Nokogiri::HTML::Document.new.fragment(doc)
-        assert_equal "<div>b</div>", fragment.to_s
+        assert_match %r%     \n<div>b</div> *%, fragment.to_s
       end
 
       def test_html_fragment_with_leading_text_and_newline
@@ -155,7 +171,7 @@ module Nokogiri
 
       def test_html_fragment_with_leading_whitespace_and_text_and_newline
         fragment = HTML::Document.new.fragment("  First line\nSecond line<br>Broken line")
-        assert_equal "First line\nSecond line<br>Broken line", fragment.to_s
+        assert_equal "  First line\nSecond line<br>Broken line", fragment.to_s
       end
 
       def test_html_fragment_with_leading_entity
@@ -179,7 +195,7 @@ module Nokogiri
       def test_to_xhtml
         doc = "<span>foo<br></span><span>bar</span>"
         fragment = Nokogiri::HTML::Document.new.fragment(doc)
-        if Nokogiri::VERSION_INFO['libxml']['loaded'] >= "2.7.0"
+        if !Nokogiri.jruby? && Nokogiri::VERSION_INFO['libxml']['loaded'] >= "2.7.0"
           assert_equal "<span>foo<br /></span><span>bar</span>", fragment.to_xhtml
         else
           assert_equal "<span>foo<br></span><span>bar</span>", fragment.to_xhtml
@@ -205,6 +221,39 @@ module Nokogiri
         assert_equal("<p>hello<!-- your ad here --></p>",
           fragment.to_s)
       end
+
+      def test_malformed_fragment_is_corrected
+        fragment = HTML::DocumentFragment.parse("<div </div>")
+        assert_equal "<div></div>", fragment.to_s
+      end
+
+      def test_unclosed_script_tag
+        # see GH#315
+        fragment = HTML::DocumentFragment.parse("foo <script>bar")
+        assert_equal "foo <script>bar</script>", fragment.to_html
+      end
+
+      def test_error_propagation_on_fragment_parse
+        frag = Nokogiri::HTML::DocumentFragment.parse "<hello>oh, hello there.</hello>"
+        assert frag.errors.any?{|err| err.to_s =~ /Tag hello invalid/}, "errors should be copied to the fragment"
+      end
+
+      def test_error_propagation_on_fragment_parse_in_node_context
+        doc = Nokogiri::HTML::Document.parse "<html><body><div></div></body></html>"
+        context_node = doc.at_css "div"
+        frag = Nokogiri::HTML::DocumentFragment.new doc, "<hello>oh, hello there.</hello>", context_node
+        assert frag.errors.any?{|err| err.to_s =~ /Tag hello invalid/}, "errors should be on the context node's document"
+      end
+
+      def test_error_propagation_on_fragment_parse_in_node_context_should_not_include_preexisting_errors
+        doc = Nokogiri::HTML::Document.parse "<html><body><div></div><jimmy></jimmy></body></html>"
+        assert doc.errors.any?{|err| err.to_s =~ /jimmy/}, "assert on setup"
+
+        context_node = doc.at_css "div"
+        frag = Nokogiri::HTML::DocumentFragment.new doc, "<hello>oh, hello there.</hello>", context_node
+        assert frag.errors.any?{|err| err.to_s =~ /Tag hello invalid/}, "errors should be on the context node's document"
+        assert frag.errors.none?{|err| err.to_s =~ /jimmy/}, "errors should not include pre-existing document errors"
+      end
     end
   end
 end
