@@ -81,6 +81,28 @@ class TestReader < Nokogiri::TestCase
     assert_equal [false, false, false, false, false, false, false],
       reader.map { |x| x.default? }
   end
+  
+  class ReallyBadIO
+    def read(size)
+      'a' * size ** 10
+    end
+  end
+
+  class ReallyBadIO4Java
+    def read(size=1)
+      'a' * size ** 10
+    end
+  end
+
+  def test_io_that_reads_too_much
+    if Nokogiri.jruby?
+      io = ReallyBadIO4Java.new
+      Nokogiri::XML::Reader(io)
+    else
+      io = ReallyBadIO.new
+      Nokogiri::XML::Reader(io)
+    end
+  end
 
   def test_in_memory
     assert Nokogiri::XML::Reader(<<-eoxml)
@@ -97,12 +119,7 @@ class TestReader < Nokogiri::TestCase
     </x>
     eoxml
     reader = Nokogiri::XML::Reader(xml)
-    if Nokogiri.ffi?
-      assert_not_nil reader.source
-      assert reader.source.is_a?(FFI::MemoryPointer)
-    else
-      assert_equal xml, reader.source
-    end
+    assert_equal xml, reader.source
   end
 
   def test_default?
@@ -405,4 +422,67 @@ class TestReader < Nokogiri::TestCase
     end
   end
 
+  def test_correct_outer_xml_inclusion
+    xml = Nokogiri::XML::Reader.from_io(StringIO.new(<<-eoxml))
+      <root-element>
+        <children>
+          <child n="1">
+            <field>child-1</field>
+          </child>
+          <child n="2">
+            <field>child-2</field>
+          </child>
+          <child n="3">
+            <field>child-3</field>
+          </child>
+        </children>
+      </root-element>
+    eoxml
+
+    nodelengths = []
+    has_child2 = []
+
+    xml.each do |node|
+      if node.node_type == Nokogiri::XML::Reader::TYPE_ELEMENT and node.name == "child"
+        nodelengths << node.outer_xml.length
+        has_child2 << !!(node.outer_xml =~ /child-2/)
+      end
+    end
+
+    assert_equal(nodelengths[0], nodelengths[1])
+    assert(has_child2[1])
+    assert(!has_child2[0])
+  end
+
+  def test_correct_inner_xml_inclusion
+    xml = Nokogiri::XML::Reader.from_io(StringIO.new(<<-eoxml))
+      <root-element>
+        <children>
+          <child n="1">
+            <field>child-1</field>
+          </child>
+          <child n="2">
+            <field>child-2</field>
+          </child>
+          <child n="3">
+            <field>child-3</field>
+          </child>
+        </children>
+      </root-element>
+    eoxml
+
+    nodelengths = []
+    has_child2 = []
+
+    xml.each do |node|
+      if node.node_type == Nokogiri::XML::Reader::TYPE_ELEMENT and node.name == "child"
+        nodelengths << node.inner_xml.length
+        has_child2 << !!(node.inner_xml =~ /child-2/)
+      end
+    end
+
+    assert_equal(nodelengths[0], nodelengths[1])
+    assert(has_child2[1])
+    assert(!has_child2[0])
+  end
 end
