@@ -24,7 +24,7 @@ module Nokogiri
 
           assert block_called
 
-          assert_equal ['a', '&b'], doc.start_elements.first.last
+          assert_equal [['foo', [['a', '&b']]]], doc.start_elements
         end
 
         def test_parser_context_yielded_in_memory
@@ -40,7 +40,7 @@ module Nokogiri
 
           assert block_called
 
-          assert_equal ['a', '&b'], doc.start_elements.first.last
+          assert_equal [['foo', [['a', '&b']]]], doc.start_elements
         end
 
         def test_xml_decl
@@ -95,6 +95,7 @@ module Nokogiri
   <foo:bar>hello world</foo:bar>
 </root>
           eoxml
+
           assert @parser.document.start_elements_namespace.length > 0
           el = @parser.document.start_elements_namespace[1]
           assert_equal 'a', el.first
@@ -144,8 +145,12 @@ module Nokogiri
         end
 
         def test_start_is_called_without_namespace
-          @parser.parse('<foo:f><bar></foo:f>')
-          assert_equal ['foo:f', 'bar'],
+          @parser.parse(<<-eoxml)
+<root xmlns:foo='http://foo.example.com/' xmlns='http://example.com/'>
+<foo:f><bar></foo:f>
+</root>
+          eoxml
+          assert_equal ['root', 'foo:f', 'bar'],
             @parser.document.start_elements.map { |x| x.first }
         end
 
@@ -168,7 +173,12 @@ module Nokogiri
             end
           end
 
-          assert_equal doc.errors.length, @parser.document.errors.length
+          # when using JRuby Nokogiri, more errors will be generated as the DOM
+          # parser continue to parse an ill formed document, while the sax parser
+          # will stop at the first error
+          unless Nokogiri.jruby?
+            assert_equal doc.errors.length, @parser.document.errors.length
+          end
         end
 
         def test_parse_with_memory_argument
@@ -245,6 +255,11 @@ module Nokogiri
           assert_raises(ArgumentError) { @parser.parse_memory(nil) }
         end
 
+        def test_bad_encoding_args
+          assert_raises(ArgumentError) { XML::SAX::Parser.new(Doc.new, 'not an encoding') }
+          assert_raises(ArgumentError) { @parser.parse_io(StringIO.new('<root/>'), 'not an encoding')}
+        end
+
         def test_ctag
           @parser.parse_memory(<<-eoxml)
             <p id="asdfasdf">
@@ -291,15 +306,50 @@ module Nokogiri
           @parser.parse_memory(<<-eoxml)
             <p id="asdfasdf">Paragraph 1</p>
           eoxml
-          assert_equal [["p", ["id", "asdfasdf"]]],
+          assert_equal [["p", [["id", "asdfasdf"]]]],
                        @parser.document.start_elements
         end
 
-        def test_parse_document
+        def test_start_element_attrs_include_namespaces
           @parser.parse_memory(<<-eoxml)
-            <p>Paragraph 1</p>
-            <p>Paragraph 2</p>
+            <p xmlns:foo='http://foo.example.com/'>Paragraph 1</p>
           eoxml
+          assert_equal [["p", [['xmlns:foo', 'http://foo.example.com/']]]],
+                       @parser.document.start_elements
+        end
+
+        def test_processing_instruction
+          @parser.parse_memory(<<-eoxml)
+            <?xml-stylesheet href="a.xsl" type="text/xsl"?>
+            <?xml version="1.0"?>
+          eoxml
+          assert_equal [['xml-stylesheet', 'href="a.xsl" type="text/xsl"']],
+                       @parser.document.processing_instructions
+        end
+
+        if Nokogiri.uses_libxml? # JRuby SAXParser only parses well-formed XML documents
+          def test_parse_document
+            @parser.parse_memory(<<-eoxml)
+              <p>Paragraph 1</p>
+              <p>Paragraph 2</p>
+            eoxml
+          end
+        end
+
+        def test_parser_attributes
+          xml = <<-eoxml
+<?xml version="1.0" ?><root><foo a="&amp;b" c="&gt;d" /></root>
+          eoxml
+
+          block_called = false
+          @parser.parse(xml) { |ctx|
+            block_called = true
+            ctx.replace_entities = true
+          }
+
+          assert block_called
+
+          assert_equal [['root', []], ['foo', [['a', '&b'], ['c', '>d']]]], @parser.document.start_elements
         end
       end
     end
