@@ -1,8 +1,33 @@
+# -*- coding: utf-8 -*-
+
 require "helper"
 
 module Nokogiri
   module XML
     class TestBuilder < Nokogiri::TestCase
+      def test_attribute_sensitivity
+        xml = Nokogiri::XML::Builder.new { |x|
+          x.tag "hello", "abcDef" => "world"
+        }.to_xml
+        doc = Nokogiri.XML xml
+        assert_equal 'world', doc.root['abcDef']
+      end
+
+      def test_builder_with_utf8_text
+        text = "test ﺵ "
+        doc = Nokogiri::XML::Builder.new(:encoding => "UTF-8") { |xml| xml.test text }.doc
+        assert_equal text, doc.content
+      end
+
+      def test_builder_escape
+        xml = Nokogiri::XML::Builder.new { |x|
+          x.condition "value < 1", :attr => "value < 1"
+        }.to_xml
+        doc = Nokogiri.XML xml
+        assert_equal 'value < 1', doc.root['attr']
+        assert_equal 'value < 1', doc.root.content
+      end
+
       def test_builder_namespace
         doc = Nokogiri::XML::Builder.new { |xml|
           xml.a("xmlns:a" => "x") do
@@ -30,14 +55,13 @@ module Nokogiri
       end
 
       def test_builder_with_unlink
-        assert_nothing_raised do
-          Nokogiri::XML::Builder.new do |xml|
-            xml.foo do
-              xml.bar { xml.parent.unlink }
-              xml.bar2
-            end
+        b = Nokogiri::XML::Builder.new do |xml|
+          xml.foo do
+            xml.bar { xml.parent.unlink }
+            xml.bar2
           end
         end
+        assert b
       end
 
       def test_with_root
@@ -87,6 +111,21 @@ module Nokogiri
         assert_equal 'bar', doc.at('foo|baz', 'foo' => 'bar').namespace.href
       end
 
+      def test_dtd_in_builder_output
+        builder = Nokogiri::XML::Builder.new do |xml|
+          xml.doc.create_internal_subset(
+                                         'html',
+                                         "-//W3C//DTD HTML 4.01 Transitional//EN",
+                                         "http://www.w3.org/TR/html4/loose.dtd"
+                                         )
+          xml.root do
+            xml.foo
+          end
+        end
+        assert_match(/<!DOCTYPE html PUBLIC "-\/\/W3C\/\/DTD HTML 4.01 Transitional\/\/EN" "http:\/\/www.w3.org\/TR\/html4\/loose.dtd">/,
+                     builder.to_xml)
+      end
+
       def test_specify_namespace_nested
         b = Nokogiri::XML::Builder.new { |xml|
           xml.root('xmlns:foo' => 'bar') do
@@ -182,13 +221,53 @@ module Nokogiri
         assert_equal 'hello', builder.doc.at('/root').content
       end
 
+      def test_raw_xml_append
+        builder = Nokogiri::XML::Builder.new do |xml|
+          xml.root do
+            xml << '<aaa><bbb/><ccc/></aaa>'
+          end
+        end
+
+        assert_equal ["aaa"],       builder.doc.at_css("root").children.collect(&:name)
+        assert_equal ["bbb","ccc"], builder.doc.at_css("aaa").children.collect(&:name)
+      end
+
+      def test_raw_xml_append_with_namespaces
+        doc = Nokogiri::XML::Builder.new do |xml|
+          xml.root("xmlns:foo" => "x", "xmlns" => "y") do
+            xml << '<Element foo:bar="bazz"/>'
+          end
+        end.doc
+
+        el = doc.at 'Element'
+        assert_not_nil el
+
+        assert_equal 'y', el.namespace.href
+        assert_nil el.namespace.prefix
+
+        attr = el.attributes["bar"]
+        assert_not_nil attr
+        assert_not_nil attr.namespace
+        assert_equal "foo", attr.namespace.prefix
+      end
+
       def test_cdata
         builder = Nokogiri::XML::Builder.new do
           root {
             cdata "hello world"
           }
         end
-        assert_equal("<?xml version=\"1.0\"?><root><![CDATA[hello world]]></root>", builder.to_xml.gsub(/\n/, ''))
+        assert_equal("<?xml version=\"1.0\"?><root><![CDATA[hello world]]></root>",
+          builder.to_xml.gsub(/\n/, ""))
+      end
+
+      def test_comment
+        builder = Nokogiri::XML::Builder.new do
+          root {
+            comment "this is a comment"
+          }
+        end
+        assert builder.doc.root.children.first.comment?
       end
 
       def test_builder_no_block
@@ -197,7 +276,8 @@ module Nokogiri
         builder.root {
           cdata string
         }
-        assert_equal("<?xml version=\"1.0\"?><root><![CDATA[hello world]]></root>", builder.to_xml.gsub(/\n/, ''))
+        assert_equal("<?xml version=\"1.0\"?><root><![CDATA[hello world]]></root>",
+          builder.to_xml.gsub(/\n/, ''))
       end
 
     private
